/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  abstractCASModule.h
///  \brief Defines an abstract class for interfacing with a computer algebra system like GAP or MAGMA
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   2010-09-29
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef ABSTRACTCASMODULE_H_
#define ABSTRACTCASMODULE_H_

#define BOOST_IOSTREAMS_NO_LIB
#undef BOOST_IOSTREAMS_WINDOWS

#include<string>
#include<vector>
#include <boost/iostreams/stream.hpp>
#include <boost/iostreams/device/file_descriptor.hpp>
#include "defs.h"

class Permutation;
template<typename RING>
class Matrix;

class AbstractCASModule
{
public:
	AbstractCASModule(char* call[]);
	virtual ~AbstractCASModule(void);

	virtual void initTrivialSn(const std::string& group, int n)=0;
	virtual void getFather(const std::string& group, std::vector<int>& father, My_Long_Int& group_order ) = 0;
	virtual void extendGroup(const std::string& group, const Permutation& newgen, int n) = 0;
	virtual void buildGroup(const std::string& group, const std::vector<Permutation>& newgens, int n)=0 ;
	virtual My_Long_Int getOrder( const std::string& group )=0;

//	virtual unsigned long getMinWords(const Matrix<Z4>& matrix, vector<Vec<Z4> >& minWords) = 0;
	
protected:
    int m_p2c[2];
    int m_c2p[2];

    typedef boost::iostreams::stream<boost::iostreams::file_descriptor_source> fdistream;
    typedef boost::iostreams::stream<boost::iostreams::file_descriptor_sink> fdostream;

    fdistream m_from;
    fdostream  m_to;

    //	boost::fdostream m_to;				///< Communication pipe to CAS.
//	boost::fdistream m_from;			///< Communication pipe from CAS.
	pid_t m_pid;						///< The process id of the CAS Child Process.
};



#endif /*ABSTRACTCASMODULE_H_*/
