/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  z4.hpp
///  \brief  Implements the arithmetic in Z_4
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   2010-09-29
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#ifndef Z4_HPP_
#define Z4_HPP_

#include "ring.hpp"
#include "ringelement.hpp"
#include "weights.hpp"

class Z4 : public Ring
{
public:
	typedef SymWeight<2> MyWeight;
    static std::string myName(void) { return "Z4"; }
	static unsigned int getSpacing(void) { return 2; };

	inline static Element<Z4> add(const Element<Z4>&  a, const Element<Z4>& b) ;
	inline static Element<Z4> addInverse(const Element<Z4>& a);
	inline static Element<Z4> minus(const Element<Z4>&  a, const Element<Z4>& b);
	inline static std::ostream& print(std::ostream& os, const Element<Z4>& a);
	inline static std::vector<Element<Z4> > multGenerator(void);

	// call the corresponding function from Ring
	inline static bool isUnit(const Element<Z4>& a);
	inline static Element<Z4> mult(const Element<Z4>& a, const Element<Z4>& b);
	inline static Element<Z4> multInverse(const Element<Z4>& a); // returns 0 if nilpotent
	inline static Element<Z4> div(const Element<Z4>& a, const Element<Z4>& b);
	inline static unsigned int height(const Element<Z4>& a) ;
	inline static bool isReducable(const Element<Z4>& a);
	inline static int compare(const Element<Z4>& a, const Element<Z4>& b);
};

Element<Z4> Z4::add(const Element<Z4>&  a, const Element<Z4>& b)
{
	return (a.getIndex()+b.getIndex())&3 ;
}

Element<Z4>  Z4::addInverse(const Element<Z4>& a)
{
	if(a.getIndex()>0)
		return (4-a.getIndex());
	return 0;
}

Element<Z4> Z4::minus(const Element<Z4>&  a, const Element<Z4>& b)
{
	return (a.getIndex()-b.getIndex()+4)&3 ;
}

std::ostream& Z4::print(std::ostream& os, const Element<Z4>& a)
{
	return os<<(int) a.getIndex();
}

std::vector<Element<Z4> > Z4::multGenerator(void)
{
	return std::vector<Element<Z4> >(1,3);
}

bool Z4::isUnit(const Element<Z4>& a)
{
	return Ring::isUnit(a.getIndex());
}
Element<Z4> Z4::mult(const Element<Z4>& a, const Element<Z4>& b)
{
	return Ring::mult(a.getIndex(), b.getIndex());
}

Element<Z4> Z4::multInverse(const Element<Z4>& a)
{
	return Ring::multInverse(a.getIndex());
}

Element<Z4> Z4::div(const Element<Z4>& a, const Element<Z4>& b)
{
	return Ring::div(a.getIndex(), b.getIndex());
}

unsigned int Z4::height(const Element<Z4>& a)
{
	return Ring::height(a.getIndex());
}

bool Z4::isReducable(const Element<Z4>& a)
{
	return Ring::isReducable(a.getIndex());
}

int Z4::compare(const Element<Z4>& a, const Element<Z4>& b)
{
	return Ring::compare(a.getIndex(), b.getIndex());
}

#endif /*Z4_HPP_*/
